FUNCTION msf_sigma, $
;INPUTS
            density, $  
            sigma_values, $ ; valeurs de bining
            u, $
            v, $
            MSK = msk
;
;
;
;
; Creation : 2001/06/05 G. Roullet <roullet@univ-brest.fr>
;
;;;
;;;
;;; Calcule la fonction de courant meridienne definie en coordonnees
;;; isopycnes
;;; i.e. calcule MSF(lat,sigma) au lieu de MSF(lat,z)
;;;
;;;

;;
;; ATTENTION ! u_s et v_s contiennent le transport par
;; unit de longueur, i.e il s'agit de u_s=u.dz intgr
;; sur la couche d'isopycne considre et non pas de la
;; vitesse moyenne dans la couche
;;
            

      print, 'Bining U ...'
      u_s = bin_velocity(density, sigma_values, u, /uu)

      print, 'Bining V ...'
      v_s = bin_velocity(density, sigma_values, v, /vv)

      print, 'Computing MSF ...'
      compute_msf, u_s, v_s, msf, MSK = msk

      return, msf
END  

FUNCTION bin_velocity, $
            dens, $
            sigma_values, $
            x1, $
            UU = uu, $
            VV = vv
;
;
@common
   
   IF keyword_set(uu) THEN BEGIN
      density = boundperio(umask*(dens+shift(dens, -1, 0, 0))/2, /uu)
      xmask = umask
   ENDIF 

   IF keyword_set(vv) THEN BEGIN
      density = boundperio(vmask*(dens+shift(dens, 0, -1, 0))/2, /vv)
      xmask = vmask
   ENDIF 

      s_s = sigma_values
      N_s = n_elements(s_s)
      N_z = jpk
      
;;
;; Definition des variables
;;
;   Profils selon les niveaux du modele (suffixe _z)
;
   c1_z = fltarr(N_z)    ; profil du contenu vertical de x
   s_z = fltarr(N_z)     ; profil de la densite
   z_zt = zdept(1:jpk)  ; profondeur au point T  (k=0 -> 5m)
   z_zw = zdepw(1:jpk)  ;                     W  (k=0 -> 10m)

;   Profils selon les couches de densite (suffixe _s)
;
   z_s  = fltarr(N_s+1)
   c1_s = fltarr(N_s+1)
   x1_s = fltarr(N_s+1)

; Tableau de sortie
;
   x1_bin = fltarr(jpi, jpj, N_s+1)

;
;  Calcul du contenu de x sur la verticale (permet d assurer la conservation integrale)
;
   x1_content = fltarr(jpi, jpj, jpk)
   x1_content(*, *, 0) = e3t(0) * x1(*, *, 0)
   FOR k = 1, jpk-1 DO x1_content(*, *, k) = x1_content(*, *, k-1) $ 
                        + e3t(k) * x1(*, *, k)

;
;  Boucle sur le domaine 2D
;
   FOR i = 0, (jpi-1) DO BEGIN 
      FOR j = 0, (jpj-1) DO BEGIN 
;
;  Indices des points T dans l ocean
;
         i_ocean = where(xmask(i, j, *) EQ 1)

         z_s = z_s*0.
         c1_s = c1_s*0.
         x1_s = x1_s*0.

         IF (i_ocean[0] NE -1) THEN BEGIN ; on n'entre que si il y a des points ocean
 
            s_z(*) = density(i, j, *)
            c1_z(*) = x1_content(i, j, *)

; critere supplementaire a imposer sur le profil pour eviter les cas
; pathologiques en attendant d'ecrire une vraie commande d'extraction
; de progils stritement croissant. Il s'agit donc d'un test adhoc.
            IF s_z(0) LT s_z(i_ocean(n_elements(i_ocean)-1)) THEN BEGIN 

;------------------------------------------------------------------------
; controle si le profil est bien strictement croissant (pour l'instant
; inutilis (avis aux amateurs)
;
;            ds = (shift(s_z, -1)-s_z)(i_ocean(0:n_elements(i_ocean)-2))
;            ds(0) = 0
;            ind = where(ds LT 0.)
;            croissant =  ind(0) EQ -1
;------------------------------------------------------------------------

            i_bottom = i_ocean(n_elements(i_ocean)-1)

            z_s(N_s) = z_zw(i_bottom)
            c1_s(N_s) = x1_content(i, j, jpk-1)
            
; extraction d'un sous profil strictement croissant
            mini = min(s_z(i_ocean))
            maxi = max(s_z(i_ocean))            

            i_min = where(s_z(i_ocean) EQ mini)
            i_max = where(s_z(i_ocean) EQ maxi)
;   on prend le plus grand des indices min
;         et le plus petit des indices max
            i_min = i_min(n_elements(i_min)-1)
            i_max = i_max[0]

;            IF i_max GE jpk-1 THEN print, i, j, i_max
;            IF i_min LE 1 THEN print, i, j, i_min
            
; Si la valeur du niveau (s_s) est plus faible que la densite de surface, 
; l isopycne est mise en surface (z_s=0)
;
            ind = where(s_s LT s_z(i_min))
            IF ind[0] NE -1 THEN BEGIN 
               z_s(ind) = 0
               c1_s(ind) = 0
            ENDIF 

; Si la valeur du niveau (s_s) est plus elevee que la densite du fond, 
; l isopycne est mise au fond (z_s=z_zw(i_bottom))
;
            ind = where(s_s GT s_z(i_max))
            IF ind[0] NE -1 THEN BEGIN 
               z_s(ind) = z_s(N_s)
               c1_s(ind) = c1_s(N_s)
            ENDIF 

            ind = where( (s_s GE s_z(i_min)) AND (s_s LT s_z(i_max)) )
            IF ind[0] NE -1 THEN BEGIN 
               
               i_profil = i_ocean(i_min:i_max)

               z_s(ind) = interpol(z_zt(i_profil), s_z(i_profil), s_s(ind))
               
;
; j'utilise la fonction spline pour interpoler le contenu
;
               c1_s(ind) = spline(z_zw(i_profil), c1_z(i_profil), z_s(ind), 1)
;
; l'interpolation lineaire marche aussi. Elle donne des resultats differents. Elle
; me semble moins propre. Je la donne en remarque.
;
;               c_s(ind+1) = interpol(c_z(i_profil), z_zw(i_profil), z_s(ind+1))
;
; Remarque : on ne divise pas par l'epaisseur de la couche
;
               x1_s(ind) = (c1_s(ind+1)-c1_s(ind));/(z_s(ind+1)-z_s(ind))

            ENDIF 
         ENDIF 
         ENDIF

         x1_bin(i, j, *) = x1_s
         
      ENDFOR 
   ENDFOR 
   
   return, x1_bin

END 


PRO compute_msf, $
           u, $
           v, $
           msf, $
           MSK = msk
;
;
;
@common
COMMON flx, indxu, indxv, indyu, indyv, zxuleng, zxvleng, zyuleng, zyvleng
   
   IF n_elements(indxu) EQ 0 THEN iniflx


;
;  jpl bandes de latitudes
;
   jpk2 = (size(u))(3) ; on remplace jpk par jpk2 = nombre de couches


   fm     = fltarr(jpl, jpk2)
   msf    = fltarr(jpl, jpk2)
   msfmsk = fltarr(jpl, jpk2)

   vert = replicate(1, jpk2)

   zu = u
   zv = v
;
;  Masque de u et v pour un calcul de msf sur le sous domaine msk(jpi,jpj)
;
   IF n_elements(msk) NE 0 THEN BEGIN 
      zumask = boundperio( (msk +shift(msk, -1, 0) ) < 1 )
      zvmask = boundperio( (msk +shift(msk, 0, -1) ) < 1 )
      zumask = zumask(*)#vert
      zvmask = zvmask(*)#vert
      zu = zu*zumask
      zv = zv*zvmask
   ENDIF 
;
;  Ecriture "tricky" optimisee... si vous avez compris comment
;  on calculait un flux a partir de champ 2D, vous avez fait le
;  plus dur : ici on generalise a 3D d''ou l''utilisation de #vert
; 
;
;  indxu pointe dans le tableau [jpi,jpj], il faut donc l''etendre
;  pour pointer dans le tableau [jpi,jpj,jpk2] et ne pas oublier
;  le decalage : le niveau k a ses indices decales de k*jpi*jpj par
;  rapport au niveau 0
;
   offset = (fltarr(180, jpl)+long(jpi)*jpj)(*)#findgen(jpk2)
   indu = indxu(*)#vert+offset
   indv = indxv(*)#vert+offset   
;
;  extension sur la verticale
;
   zuleng = zxuleng(*)#vert
   zvleng = zxvleng(*)#vert
;
;  calcul du flux
;
   z = reform(zu(indu)*zuleng+zv(indv)*zvleng, 180, jpl, jpk2)
;
;  integration zonale du flux !
;
   fm = total(z, 1) 
;
;  calcul de la msf en integrant depuis le fond
;
;        Remarque : on ne multiplie pas par e3t car les champs
;        d'entree sont deja integrs verticalement (sur l'epaisseur de
;        couche).
;
   FOR k = jpk2-2, 0, -1 DO msf(*, k) = msf(*, k+1)-fm(*, k); *e3t(k)

END 
