PRO plt_def
;
; Defaults plot attributes
;
@com_eg
@common

; boxes used in horizontal/vertical display
; -----------------------------------------
;
; horizontal domain

;   box_h = [20,380,-30,30]
;   box_h = [100,290,-30,30]
;   box_h = [10,180,-20,20]
;   box_h = [180,270,-30,30]
;   box_h = [0,360,-90,90]
   box_h = [20,380,-30,30]
;   box_h = [20,380,-45,45]
;   box_h = [20,380,-90,-50]
;   box_h = [20,380,-90,90]
;   box_h = [20,150,0,30]
;   box_h = [100,290,-40,40]
;   box_h = [180,290,-5,10]

   glamboundary_box = [20, 380]

; vertical domain

   depth_z = 5000 
   zoom_z = 400

   hpa_min = 10
   hpa_max = 500
   
   msf_mean =  0
; vertical average for 3D fields 

;   vert_type = 'level'          ;  'z' for depth/altitude or 'level' or '0' for nothing
   vert_type = '0'
   vert_mean = [1, 1]   ; [depth1,depth2] or [level1,level2] in C notation 0-jpk-1 

; density domain (sigma) + delta sigma

   sig_min = 20.
   sig_max = 27.
   sig_del = 0.2

   sig_bowl =  0   ; 0/1 bowl overlay on density binned plots

; Statistics
; -----------

;  max date in spectrums (in days)
;  spectrum window (in days)
;  tukey window (in years)
;  bootstrap window to compute error bar for 1d time serie (in units of time array)

   max_spec = 7*360
   spec_win = long(20)*long(360)
   tukey = 20
   boot_win = [24, 36, 48]

; cross-correlation and normalisation:
   c_normal = 0       ; normalise data serie
   c_correl =  1      ; when overlay of 1d curves : 0/1 
   lag_correl = 12L    ; bound for lagged-correlation [-lag_correl,..,0,...,lag_correl]

; standard deviation
   stddev_diff = 0       ; 0/1 (if = 1 makes a difference between the
                         ; std dev for a given month and the std dev of the whole time serie)
                         ; Active with the make_stddev macro for a given month (ex : @@sosstdev_01 for
                         ; sst std dev only for January)


; axis
; ----
;  lat_axis : latitudinal axis : 'reg'ular or 'sin'us latitude
   lat_axis = 'reg'


; min /max
; --------
;  free minmax in 1D-plots (yes) or take fld_glo_mmx.def value (no) [yes]
   free_1d_minmax = 'no'
;   free_1d_minmax = 'yes'

; Hovmoellers
; -----------
;  trend_typ : 0 no trend
;              1 remove initial value to serie (trend)
;              2 remove previous value to current (drift)
;              3[<n>] remove serie to mean of <n> final values (inverse trend)
;              4[<n>] remove [n]-average running mean of serie (anomaly)
;                   for monthly serie use <n>=12 to remove mean
;                   seasonal cycle
;      this value can be field-overriden by specifying @t<trend_typ> after
;      timave in post-it line
;              6[<n>] time integral of field activation. <n> is the
;              number of previous time steps to integrate from
;              (default is from beginning of time serie)
;
;
;  field_int : field time integral (0/1)
;  nb_cycles : number of repeated cycles for seasonal time series
;  def_stride : default stride in hovmoeller sampling (see domain_boxes.def)

   trend_typ = '0'
   field_int = 0
   nb_cycles = 2
   def_stride = 1
   asciidir = homedir+'out/ascii_out/'
;
;  calendar type
;          0: 365 days/year
;          1: gregorian
;          N: N days per month
;
   calendar_type = 30
;
; Graphic options
; ----------------
; shadind : (fill) 1/0
; pal_type: 'bw', 'col', '2dom' (2dom = read 2 domains in isolignes - ISOSP)
; grey_shade: % of black for 2dom case
; grey_shade_1/2: % of black for 2dom case : difference plots (-/+)
; col_palette: 'yes' or 'no' (put colorbar or not)
;
   shading = 1
   pal_type = 'col'
   grey_shade = 20
   grey_shade_1 = 5
   grey_shade_2 = 20
   col_palette = 'yes'
   cont_fill = 1         ; continent fill for atmosphere grids (1/0)
   cont_real = 0         ; draw real continents (0)
                         ; = 0 (no real continents drawn)
                         ; = 1 (real continents drawn instead of the mask)
                         ; = 2 (mask + real continents drawn)

   vector_sample = 2     ; vector sampling (1)
;;
   line_thick = [2,2,2,2,2,2,2,2]   ; 1D plot line thickness (for each overlay curves)
   line_style = [1,1,1,1,2,2,3,3]   ; 1D plot line style (for each overlay curves)
   line_color = [1,2,3,4,6,7,7,8]   ; 1D plot line color (for each overlay curves)
; 1=black; 2=red; 3=green; 4=blue; 6=purple
; 43 ---- 90 ---- 160 ---- 190 ---- 210 ---- 254
; blue    turq    green    yell     orange   red
;;
   symbol_families = '1'         ; 4x3 number of colors in time symbol plots (modulo)
                                ; use <n> or <n>x<m> (n same colors, modulo nxm)
   symbol_families = '4x3'
;;   symbol_families = '12x1'
   symbol_style = [1,1,1,1,2,2,1,1,1,1,1,1]   ; Symbol style (for each overlay symbol)
   symbol_color = [1,2,3,4,2,3,7,8,9,10,11,12]   ; Symbol line color (for each overlay symbol)
   mean_sc_only = 0
;;   mean_sc_only = 4   ; 1 = Only plot mean SC in yfx when symbol_families='4x3' and hotyp=t
                      ; 2 = Only plot SC of std dev when symbol_families='4x3' and hotyp=t
                      ;      (require 1m@t412)
                      ; 4 = Only plot ICS per month + errorbar when symbol_families='12x1'


   langage = 'gb'        ; Langage (fr/gb)
;
; Graphic keywords for axis, contours
;
;;   contour_options = ',/nocontour'	;;;;;; include this line to delete contour labels
   contour_options = ',cell_fill=2'     ;;;;;; include this line to include contour label   
   look = ',linestyle=0,xthick=2,ythick=2,zthick=2,sepdate='' '''

; Use of page space and titles
; marge_option : add or reduce marge around the plot ; suitable for
;                optimizing space when several plots on the same
;                window. See title_type and titles keyword below to
;                set it
   fill_space = 1        ; fill space on plot (0/1) (/rempli option) ?
   marge_option = 'marge=[0,0,2,2]' ; (use [0, 0, -2, -2] to fill up even more space
   title_type = 'TS'      ; default type of titles 
                         ; 'T' for title only, 'S' for subtitle only
                         ; 'TS' for both 
                         ; 'off' for no titles
   

   default_txt_format = 'E' ; default text in legend for 1D plots (E)xperiment(V)ariable[L]ong name[U]nits(B)ox

; Multiple windows (0/1) turn on for one window per plot

   multi_win = 0

; Save PostScript files

   save_ps = 1

; Averages
; ---------
; land-sea mask use in atmospheric boxes  0 : take all
;                                         1 : ocean points
;                                         2 : land points
;
   atmos_msk = 1
;
; Data domain (OPA only)
; -----------
   h_config_oce = 'ORCA_R2'
;   h_config_oce = 'ORCA_R4'
;   h_config_oce = '';   data_domain = 'pacific'
   data_domain = 'global'
;   data_domain = 'zonal'

   v_config_oce =  'L30'
;   v_config_oce =  ''
   orca_mask_version =  'V3' ;V1(ST7/CT runs), V2 (? runs), V3 (CH, Dan runs, CM4)
; Zoomed domains (indexes are within array [0:181,0:148])
;   global        <file>_grid_<grd>.nc                               27118 pts
;   equator       <file>_grid_<grd>_eq.nc  (j=73)                      182 pts
;   equator_band  <file>_grid_<grd>_eqb.nc  (j=64,82 -> [5S,5N])       2912 pts = 10%
;   pacific       <file>_grid_<grd>_pac.nc  (j=49,97 -> [30S,30N],
;                                            i=16,106 -> [110E,70W])   4320 pts = 16%
;                                            
;   pacific_eq    <file>_grid_<grd>_paceq.nc  (j=64,82 -> [5S,5N],
;                                              i=16,106 -> [110E,70W]) 1620 pts = 6%
;   zonal         <file>_grid_<grd>_diaznl.nc
;
;  data_domain = 'glosea'
;   data_domain = 'equator'
   data_domain = 'equator_band'
;   data_domain = 'pacific_eq'
   data_domain = 'zonal'
   data_domain = 'pacific'
   data_domain = 'global'

; grids list (IPCC atmos regular)
   nc_grids_list = ['ct42', 'mt42', 'mt106', 'nt85', 'giss4x5', 'giss4x3', 'cgcm3_1', 'mk3', 'gfdl2.5x2', 'fgoals1', 'inmcm3', 'cgcm232', 'nt42', 'n96', 'mt63', 'lmdzl', 'lmdzh', 'ncpt62', 'ert106', 'it30', 'bt42', 'reg1']

; machine type ('x' or 'WIN')
	dev_type='x'

; debug mode
        debug_w = 1

END
